#include <ESP8266WiFi.h>  //INCLUDING THE ESP8266 LIBRARY
#include "MegunoLink.h"
#include "CommandHandler.h"

//FUNCTION DEFINITIONS
void CONFIGURE_PINS();
void GPIO_INITIAL_STATE();
void CONNECT_TO_WIFI();
void ESTABLISH_TCP_CONNECTION();
void SEND_GET_REQUEST();
void WAIT_FOR_SERVER();
void PRINT_API_RESPONSE();
float PARSE_TEMPERATURE();
void FLUSH_RECEIVE_BUFFER();
void CLOSE_TCP_CONNECTION();
float CONVERT_CELSIUS_TO_FAHRENHEIT(float);
void GET_MODULE_VOLTAGE();
int GET_RED_LED_INTENSITY(int, float);
int GET_GREEN_LED_INTENSITY(int, float);
int GET_BLUE_LED_INTENSITY(int, float);
int GET_RANDOM_RED_LED_INTENSITY();
int GET_RANDOM_GREEN_LED_INTENSITY();
int GET_RANDOM_BLUE_LED_INTENSITY();
void LOAD_RGB_LED(int, int, int);
void FADE_TO_COLOR_OF_TEMPERATURE(int, int, int);
void FADE_TO_RANDOM_COLOR(int, int, int);
void CHANGE_TO_WIFI_MODE(CommandParameter);
void CHANGE_TO_SET_TEMP_MODE(CommandParameter);
void CHANGE_TO_SET_RGB_MODE(CommandParameter);
void CHANGE_TO_RANDOM_MODE(CommandParameter);
void CHANGE_TO_WHITE_MODE(CommandParameter);
void SET_TEMPERATURE(CommandParameter);
void SET_RED_INTENSITY(CommandParameter);
void SET_GREEN_INTENSITY(CommandParameter);
void SET_BLUE_INTENSITY(CommandParameter);
void UPDATE_GUI();

//WIFI CONFIGURATION VARIABLES
const char INTERNET_SSID[] = ""; //CREATED VARIABLE FOR SSID
const char SSID_PASSWORD[] = "";  //PASSWORD FOR SSID

//WEATHER LOCATION VARIABLES
const String CITY_ID = "";

//API CONFIGURATION VARIABLE
const char HOST[] = "api.openweathermap.org";
const int PORT = 80;
const String API_KEY = "";

//PIN IDENTIFICATION VARIABLES
const int RED_LED = D7;
const int GREEN_LED = D6;
const int BLUE_LED = D5;

const int GREEN_INDICATOR_LED = D2;
const int RED_INDICATOR_LED = D0;
const int YELLOW_INDICATOR_LED = D4;

const int INPUT_VOLTAGE_MEAS = A0;

const int UNUSED_PIN_D1 = D1;
const int UNUSED_PIN_D3 = D3;
const int UNUSED_PIN_D8 = D8;

//DELAY VARIABLES
unsigned long TIMEOUT = 10000;  //DELAY FOR 10 SECONDS
unsigned long TEMPERATURE_UPDATE_DELAY = 10*60;
unsigned long TEMPERATURE_UPDATE_DELAY_COUNT = 0;

const int FADE_TO_RANDOM_COLOR_DELAY = 1000;
const int FADE_TO_RANDOM_COLOR_CYCLES = 300;
int FADE_TO_RANDOM_COLOR_CYCLES_COUNT = 0;

//OTHER VARIABLES
int WIFI_CONNECT_COUNT = 0;
int TEMPERATURE_UPDATES = 0;
unsigned long TIMESTAMP;

//LED INTENSITY VARIABLES
int RED_LED_INTENSITY_INT = 0;
int GREEN_LED_INTENSITY_INT = 0;
int BLUE_LED_INTENSITY_INT = 0;

float RED_LED_INTENSITY_FLOAT = 0;
float GREEN_LED_INTENSITY_FLOAT = 0;
float BLUE_LED_INTENSITY_FLOAT = 0;

//TEMPERATURE VARIABLES
int  TEMPERATURE_INT = 0;
float TEMPERATURE_FLOAT = 0;

//MODULE VOLTAGE VARIABLES
int MODULE_VOLTAGE_INT;
float MODULE_VOLTAGE_FLOAT;
float MODULE_VOLTAGE_SCALED;

//LOGIC VARIABLES
bool TEMPERATURE_DATA_GOOD = false;
boolean LEDState = false;

//GUI VARIABLES
String CURRENT_MODE = "WIFI MODE";
int SET_TEMPERATURE_INT = 0;
float SET_TEMPERATURE_FLOAT = 0.0;
int SET_TEMPERATURE_RED_INTENSITY = 0;
int SET_TEMPERATURE_GREEN_INTENSITY = 0;
int SET_TEMPERATURE_BLUE_INTENSITY = 0;
int SET_RGB_RED_INTENSITY_INT = 0;
int SET_RGB_GREEN_INTENSITY_INT = 0;
int SET_RGB_BLUE_INTENSITY_INT = 0;

//WIFI CLIENT
WiFiClient client;

CommandHandler<> SerialCommandHandler;

void setup() {
  
  Serial.begin(57600); //START SERIAL BUS AT A BAUD RATE OF 9600BPS
  delay(100); //SHORT 100MS DELAY

  CONFIGURE_PINS(); //CONFIGURES GPIO TO INPUTS AND OUTPUTS
  GPIO_INITIAL_STATE(); //CONFIGURES GPIO TO INITIAL STATE

  CONNECT_TO_WIFI();  //CONNECTS TO WIFI USING SSID AND PASSWORD

  SerialCommandHandler.AddCommand(F("WIFI_MODE_BUTTON"), CHANGE_TO_WIFI_MODE);
  SerialCommandHandler.AddCommand(F("SET_TEMP_MODE_BUTTON"), CHANGE_TO_SET_TEMP_MODE);
  SerialCommandHandler.AddCommand(F("SET_RGB_MODE_BUTTON"), CHANGE_TO_SET_RGB_MODE);
  SerialCommandHandler.AddCommand(F("RANDOM_MODE_BUTTON"), CHANGE_TO_RANDOM_MODE);
  SerialCommandHandler.AddCommand(F("WHITE_MODE_BUTTON"), CHANGE_TO_WHITE_MODE);
  SerialCommandHandler.AddCommand(F("SET_TEMPERATURE_TRACKBAR"), SET_TEMPERATURE); // Attach command
  SerialCommandHandler.AddCommand(F("SET_RED_TRACKBAR"), SET_RED_INTENSITY); // Attach command
  SerialCommandHandler.AddCommand(F("SET_GREEN_TRACKBAR"), SET_GREEN_INTENSITY); // Attach command
  SerialCommandHandler.AddCommand(F("SET_BLUE_TRACKBAR"), SET_BLUE_INTENSITY); // Attach command
}

void loop() {

  SerialCommandHandler.Process();

  if (CURRENT_MODE == "WIFI MODE"){
      
    if (TEMPERATURE_UPDATES == 11){
      WiFi.disconnect();
      delay(500);
      CONNECT_TO_WIFI();
      TEMPERATURE_UPDATES = 0;
    }
  
    ESTABLISH_TCP_CONNECTION(); //ESTABLISH TCP CONNECTION
    SEND_GET_REQUEST(); //SEND GET REQUEST
    WAIT_FOR_SERVER();  //WAIT FOR RESPONSE FROM SERVER
    TEMPERATURE_FLOAT = PARSE_TEMPERATURE();  //PARSE TEMPERATURE
    
    TEMPERATURE_INT = TEMPERATURE_FLOAT;  //UPDATE FLOATING TEMPERATURE VARIABLE  
    FLUSH_RECEIVE_BUFFER(); //FLUSH RECEIVE BUFFER
    CLOSE_TCP_CONNECTION(); //CLOSE TCP CONNECTION
  
    if (TEMPERATURE_DATA_GOOD){
      FADE_TO_COLOR_OF_TEMPERATURE(RED_LED_INTENSITY_INT, GREEN_LED_INTENSITY_INT, BLUE_LED_INTENSITY_INT);
    }
  
    if (!TEMPERATURE_DATA_GOOD){
      while(FADE_TO_RANDOM_COLOR_CYCLES_COUNT <= FADE_TO_RANDOM_COLOR_CYCLES){
        FADE_TO_RANDOM_COLOR(RED_LED_INTENSITY_INT, GREEN_LED_INTENSITY_INT, BLUE_LED_INTENSITY_INT);
        FADE_TO_RANDOM_COLOR_CYCLES_COUNT = FADE_TO_RANDOM_COLOR_CYCLES_COUNT + 1;
        UPDATE_GUI();
        delay(FADE_TO_RANDOM_COLOR_DELAY);
      }
      FADE_TO_RANDOM_COLOR_CYCLES_COUNT = 0;
    }
  
    Serial.println();
  
    TEMPERATURE_UPDATES = TEMPERATURE_UPDATES + 1;
    UPDATE_GUI();
    
    while ((TEMPERATURE_UPDATE_DELAY_COUNT < TEMPERATURE_UPDATE_DELAY)&&(CURRENT_MODE == "WIFI MODE")){
      TEMPERATURE_UPDATE_DELAY_COUNT = TEMPERATURE_UPDATE_DELAY_COUNT + 1;
      delay(1000);
      SerialCommandHandler.Process();
    }
    TEMPERATURE_UPDATE_DELAY_COUNT = 0;
  }

  if (CURRENT_MODE == "SET TEMP MODE"){
    SET_TEMPERATURE_FLOAT = SET_TEMPERATURE_INT;
    SET_TEMPERATURE_RED_INTENSITY = GET_RED_LED_INTENSITY(SET_TEMPERATURE_INT, SET_TEMPERATURE_FLOAT);
    SET_TEMPERATURE_GREEN_INTENSITY = GET_GREEN_LED_INTENSITY(SET_TEMPERATURE_INT, SET_TEMPERATURE_FLOAT);
    SET_TEMPERATURE_BLUE_INTENSITY = GET_BLUE_LED_INTENSITY(SET_TEMPERATURE_INT, SET_TEMPERATURE_FLOAT);
    LOAD_RGB_LED(SET_TEMPERATURE_RED_INTENSITY, SET_TEMPERATURE_GREEN_INTENSITY, SET_TEMPERATURE_BLUE_INTENSITY); 
  }

  if (CURRENT_MODE == "SET RGB MODE"){
    RED_LED_INTENSITY_INT = SET_RGB_RED_INTENSITY_INT;
    RED_LED_INTENSITY_FLOAT = RED_LED_INTENSITY_INT;
    GREEN_LED_INTENSITY_INT = SET_RGB_GREEN_INTENSITY_INT;
    GREEN_LED_INTENSITY_FLOAT = GREEN_LED_INTENSITY_INT;
    BLUE_LED_INTENSITY_INT = SET_RGB_BLUE_INTENSITY_INT;
    BLUE_LED_INTENSITY_FLOAT = BLUE_LED_INTENSITY_INT;
    LOAD_RGB_LED(SET_RGB_RED_INTENSITY_INT, SET_RGB_GREEN_INTENSITY_INT, SET_RGB_BLUE_INTENSITY_INT);    
  }

  if (CURRENT_MODE == "RANDOM MODE"){
    FADE_TO_RANDOM_COLOR(RED_LED_INTENSITY_INT, GREEN_LED_INTENSITY_INT, BLUE_LED_INTENSITY_INT);    
  }

  if (CURRENT_MODE == "WHITE MODE"){
    RED_LED_INTENSITY_INT = 1023;
    GREEN_LED_INTENSITY_INT = 1023;
    BLUE_LED_INTENSITY_INT = 1023;
    RED_LED_INTENSITY_FLOAT = RED_LED_INTENSITY_INT;
    GREEN_LED_INTENSITY_FLOAT = GREEN_LED_INTENSITY_INT;
    BLUE_LED_INTENSITY_FLOAT = BLUE_LED_INTENSITY_INT;
    LOAD_RGB_LED(RED_LED_INTENSITY_INT, RED_LED_INTENSITY_INT, RED_LED_INTENSITY_INT);    
  }  
  UPDATE_GUI();  
}

//CONFIGURES GPIO TO INPUTS AND OUTPUTS
void CONFIGURE_PINS(){
  pinMode(RED_LED, OUTPUT);
  pinMode(GREEN_LED, OUTPUT);
  pinMode(BLUE_LED, OUTPUT);
  
  pinMode(GREEN_INDICATOR_LED, OUTPUT);
  pinMode(RED_INDICATOR_LED, OUTPUT);
  pinMode(YELLOW_INDICATOR_LED, OUTPUT);
  
  pinMode(UNUSED_PIN_D1, OUTPUT);
  pinMode(UNUSED_PIN_D3, INPUT);
  pinMode(UNUSED_PIN_D8, INPUT);

  pinMode(INPUT_VOLTAGE_MEAS, INPUT);  
}

//CONFIGURES GPIO TO INITIAL STATE
void GPIO_INITIAL_STATE(){
  digitalWrite(RED_LED, LOW);
  digitalWrite(GREEN_LED, LOW);
  digitalWrite(BLUE_LED, LOW);

  digitalWrite(GREEN_INDICATOR_LED, LOW);
  digitalWrite(RED_INDICATOR_LED, LOW);
  digitalWrite(YELLOW_INDICATOR_LED, HIGH);
  
  digitalWrite(UNUSED_PIN_D1, LOW);
}

//CONNECTS TO WIFI USING SSID AND PASSWORD
void CONNECT_TO_WIFI(){
  unsigned long WIFI_CONNECTION_TIME_MILLIS = millis();
  unsigned long PREVIOUS_WIFI_CONNECTION_TIME_MILLIS = WIFI_CONNECTION_TIME_MILLIS;
  int WIFI_CONNECTION_TIMEOUT_TIME = 1*60*1000;
  
  Serial.print("CONNECTING TO ");
  Serial.println(INTERNET_SSID);
  WiFi.begin(INTERNET_SSID, SSID_PASSWORD);

    while((WiFi.status() != WL_CONNECTED) && (WIFI_CONNECTION_TIME_MILLIS-PREVIOUS_WIFI_CONNECTION_TIME_MILLIS<WIFI_CONNECTION_TIMEOUT_TIME)){
      delay(200);
      Serial.print(".");
  
      digitalWrite(GREEN_INDICATOR_LED, LOW);
      digitalWrite(YELLOW_INDICATOR_LED, HIGH);
      digitalWrite(RED_INDICATOR_LED, LOW);
  
      if(WIFI_CONNECT_COUNT == 0){
        digitalWrite(GREEN_INDICATOR_LED, HIGH);      
        digitalWrite(YELLOW_INDICATOR_LED, HIGH);
        digitalWrite(RED_INDICATOR_LED, LOW);
      }    
      if(WIFI_CONNECT_COUNT == 1){
        digitalWrite(GREEN_INDICATOR_LED, LOW);      
        digitalWrite(YELLOW_INDICATOR_LED, LOW);
        digitalWrite(RED_INDICATOR_LED, LOW);
      }
      if(WIFI_CONNECT_COUNT == 2){
        digitalWrite(GREEN_INDICATOR_LED, LOW);      
        digitalWrite(YELLOW_INDICATOR_LED, HIGH);
        digitalWrite(RED_INDICATOR_LED, HIGH);
      }
  
      WIFI_CONNECT_COUNT = WIFI_CONNECT_COUNT + 1;
  
      if(WIFI_CONNECT_COUNT > 3){
        WIFI_CONNECT_COUNT = 0;
      }
      
      WIFI_CONNECTION_TIME_MILLIS = millis(); 
  }

  if (WiFi.status() == WL_CONNECTED){

    digitalWrite(GREEN_INDICATOR_LED, LOW);
    digitalWrite(YELLOW_INDICATOR_LED, LOW);
    digitalWrite(RED_INDICATOR_LED, LOW);  
  
    Serial.println();
    Serial.println("WIFI CONNECTION SUCCESS");
    Serial.println();

    TEMPERATURE_DATA_GOOD = true;
  }

  else if (WiFi.status() != WL_CONNECTED){
    digitalWrite(GREEN_INDICATOR_LED, LOW);
    digitalWrite(YELLOW_INDICATOR_LED, HIGH);
    digitalWrite(RED_INDICATOR_LED, HIGH);  
  
    Serial.println();
    Serial.println("WIFI CONNECTION FAILED");
    Serial.println();

    TEMPERATURE_DATA_GOOD = false;
  }  
}

//ESTABLISH TCP CONNECTION
void ESTABLISH_TCP_CONNECTION(){
  Serial.print("CONNECTING TO ");
  Serial.println(HOST);

  if(!client.connect(HOST, PORT)){
    Serial.println("CONNECTION FAILED");
    digitalWrite(RED_INDICATOR_LED, HIGH);
    digitalWrite(GREEN_INDICATOR_LED, LOW);
    TEMPERATURE_DATA_GOOD = false;
  }

  else{
    Serial.println("CONNECTION SUCCESS");
    digitalWrite(RED_INDICATOR_LED, LOW);
    digitalWrite(GREEN_INDICATOR_LED, HIGH);
  }}

//SEND GET REQUEST
void SEND_GET_REQUEST(){
  client.println("GET /data/2.5/weather?id="+CITY_ID+"&units=metric&APPID="+API_KEY);
  client.println("Host: api.openweathermap.org");
  client.println("User-Agent: ArduinoWiFi/1.1");
}

//WAIT FOR RESPONSE FROM SERVER
void WAIT_FOR_SERVER(){
  delay(500);
  TIMESTAMP = millis();
  while (!client.available() && (millis() < TIMESTAMP + TIMEOUT)){
    delay(1);
  }  
}

//PRINTS API RESPONSE
void PRINT_API_RESPONSE(){
  String result;

  while (client.connected() || client.available())    
       {                                             //connected or data available
         char c = client.read();                     //gets byte from ethernet buffer
         result = result+c;
       }

  client.stop();                                      //stop client
  result.replace('[', ' ');
  result.replace(']', ' ');
  Serial.println(result);
}

//PARSE TEMPERATURE
float PARSE_TEMPERATURE(){
  if (client.find("temp\":")){
    TEMPERATURE_FLOAT = client.parseFloat();
    TEMPERATURE_FLOAT = CONVERT_CELSIUS_TO_FAHRENHEIT(TEMPERATURE_FLOAT);
    Serial.println("PARSE TEMPERATURE SUCCESS");
    Serial.print("TEMPERATURE: ");
    Serial.print(TEMPERATURE_FLOAT);
    Serial.println(" F");
    digitalWrite(RED_INDICATOR_LED, LOW);
    digitalWrite(GREEN_INDICATOR_LED, HIGH);
    TEMPERATURE_DATA_GOOD = true;
    return TEMPERATURE_FLOAT;
  }
  else{
    Serial.println("PARSE TEMPERATURE FAILED");
    digitalWrite(RED_INDICATOR_LED, HIGH);
    digitalWrite(GREEN_INDICATOR_LED, LOW);
    TEMPERATURE_DATA_GOOD = false;
  }
}

//FLUSH RECEIVE BUFFER
void FLUSH_RECEIVE_BUFFER(){
  while (client.available()){
    client.readStringUntil('\r');
  }
}

//CLOSE TCP CONNECTION
void CLOSE_TCP_CONNECTION(){ 
  client.stop();
  Serial.println("CONNECTION CLOSED");
  Serial.println();

  digitalWrite(YELLOW_INDICATOR_LED, HIGH);
  delay(200);
  digitalWrite(YELLOW_INDICATOR_LED, LOW);
  delay(200);
  digitalWrite(YELLOW_INDICATOR_LED, HIGH);
  delay(200);
  digitalWrite(YELLOW_INDICATOR_LED, LOW);
}

float CONVERT_CELSIUS_TO_FAHRENHEIT(float TEMPERATURE_C){
  float TEMPERATURE_F;
  TEMPERATURE_F = (TEMPERATURE_C*9.0/5.0)+32.0;
  return TEMPERATURE_F;
}

int GET_RED_LED_INTENSITY(int TEMPERATURE_INT, float TEMPERATURE_FLOAT){  
  Serial.println(TEMPERATURE_INT);
  if(TEMPERATURE_INT <= 0){
    RED_LED_INTENSITY_FLOAT = 0.0;  //CALCULATE RED LED INTENSITY
  }
  else if(TEMPERATURE_INT>0 && TEMPERATURE_INT<50){
    RED_LED_INTENSITY_FLOAT = 0.0;  //CALCULATE RED LED INTENSITY
  }
  else if(TEMPERATURE_INT == 50){
    RED_LED_INTENSITY_FLOAT = 0.0;  //CALCULATE RED LED INTENSITY
  }
  else if(TEMPERATURE_INT>50 && TEMPERATURE_INT<100){
    RED_LED_INTENSITY_FLOAT = 1023.0/50.0*TEMPERATURE_FLOAT-1023.0;  //CALCULATE RED LED INTENSITY 
    Serial.println("HELLO 1");   
  }
  else if(TEMPERATURE_INT >= 100){
    RED_LED_INTENSITY_FLOAT = 1023.0;  //CALCULATE RED LED INTENSITY
    Serial.println("HELLO 2");  
  }  

  Serial.print("RED LED DUTY CYCLE = ");
  Serial.print(RED_LED_INTENSITY_FLOAT/1023.0*100.0);
  Serial.println("%");
  
  RED_LED_INTENSITY_INT = RED_LED_INTENSITY_FLOAT;
  
  return RED_LED_INTENSITY_INT; //CALCULATE RED LED INTENSITY
}

int GET_GREEN_LED_INTENSITY(int TEMPERATURE_INT, float TEMPERATURE_FLOAT){  
  if(TEMPERATURE_INT <= 0){
    GREEN_LED_INTENSITY_FLOAT = 0.0;  //CALCULATE GREEN LED INTENSITY
  }
  else if(TEMPERATURE_INT>0 && TEMPERATURE_INT<50){
    GREEN_LED_INTENSITY_FLOAT = 1023.0/50.0*TEMPERATURE_FLOAT;  //CALCULATE GREEN LED INTENSITY
  }
  else if(TEMPERATURE_INT == 50){
    GREEN_LED_INTENSITY_FLOAT = 1023.0;  //CALCULATE GREEN LED INTENSITY
  }
  else if(TEMPERATURE_INT>50 && TEMPERATURE_INT<100){
    GREEN_LED_INTENSITY_FLOAT = -1023.0/50.0*TEMPERATURE_FLOAT+2046.0;  //CALCULATE GREEN LED INTENSITY    
  }
  else if(TEMPERATURE_INT >= 100){
    GREEN_LED_INTENSITY_FLOAT = 0.0;  //CALCULATE GREEN LED INTENSITY
  }  

  Serial.print("GREEN LED DUTY CYCLE = ");
  Serial.print(GREEN_LED_INTENSITY_FLOAT/1023.0*100.0);
  Serial.println("%");

  GREEN_LED_INTENSITY_INT = GREEN_LED_INTENSITY_FLOAT;
  
  return GREEN_LED_INTENSITY_INT; //CALCULATE GREEN LED INTENSITY
}

int GET_BLUE_LED_INTENSITY(int TEMPERATURE_INT, float TEMPERATURE_FLOAT){  
  if(TEMPERATURE_INT <= 0){
    BLUE_LED_INTENSITY_FLOAT = 1023.0;  //CALCULATE BLUE LED INTENSITY
  }
  else if(TEMPERATURE_INT>0 && TEMPERATURE_INT<50){
    BLUE_LED_INTENSITY_FLOAT = -1023.0/50.0*TEMPERATURE_FLOAT+1023.0;  //CALCULATE BLUE LED INTENSITY
  }
  else if(TEMPERATURE_INT == 50){
    BLUE_LED_INTENSITY_FLOAT = 0.0;  //CALCULATE BLUE LED INTENSITY
  }
  else if(TEMPERATURE_INT>50 && TEMPERATURE_INT<100){
    BLUE_LED_INTENSITY_FLOAT = 0.0;  //CALCULATE BLUE LED INTENSITY    
  }
  else if(TEMPERATURE_INT >= 100){
    BLUE_LED_INTENSITY_FLOAT = 0.0;  //CALCULATE BLUE LED INTENSITY
  }  

  Serial.print("BLUE LED DUTY CYCLE = ");
  Serial.print(BLUE_LED_INTENSITY_FLOAT/1023.0*100.0);
  Serial.println("%");
  
  BLUE_LED_INTENSITY_INT = BLUE_LED_INTENSITY_FLOAT;
  
  return BLUE_LED_INTENSITY_INT; //CALCULATE BLUE LED INTENSITY
}

int GET_RANDOM_RED_LED_INTENSITY(){  
  RED_LED_INTENSITY_INT = random(0,1023);
  RED_LED_INTENSITY_FLOAT = RED_LED_INTENSITY_INT;
  Serial.print("RED LED DUTY CYCLE = ");
  Serial.print(RED_LED_INTENSITY_FLOAT/1023.0*100.0);
  Serial.println("%");
  
  return RED_LED_INTENSITY_INT;
}

int GET_RANDOM_GREEN_LED_INTENSITY(){  
  GREEN_LED_INTENSITY_INT = random(0,1023);
  GREEN_LED_INTENSITY_FLOAT = GREEN_LED_INTENSITY_INT;
  Serial.print("GREEN LED DUTY CYCLE = ");
  Serial.print(GREEN_LED_INTENSITY_FLOAT/1023.0*100.0);
  Serial.println("%");
  
  return GREEN_LED_INTENSITY_INT;
}

int GET_RANDOM_BLUE_LED_INTENSITY(){  
  BLUE_LED_INTENSITY_INT = random(0,1023);
  BLUE_LED_INTENSITY_FLOAT = BLUE_LED_INTENSITY_INT;
  Serial.print("BLUE LED DUTY CYCLE = ");
  Serial.print(BLUE_LED_INTENSITY_FLOAT/1023.0*100.0);
  Serial.println("%");
  
  return BLUE_LED_INTENSITY_INT;
}

void LOAD_RGB_LED(int RED_LED_INTENSITY_INT,int GREEN_LED_INTENSITY_INT, int BLUE_LED_INTENSITY_INT){
  analogWrite(RED_LED, RED_LED_INTENSITY_INT);  //SET RED LED TO RED LED INTENSITY
  analogWrite(GREEN_LED, GREEN_LED_INTENSITY_INT);  //SET GREEN LED TO BLUE LED INTENSITY
  analogWrite(BLUE_LED, BLUE_LED_INTENSITY_INT);  //SET BLUE LED TO BLUE LED INTENSITY
}

void GET_MODULE_VOLTAGE(){
  MODULE_VOLTAGE_INT = analogRead(INPUT_VOLTAGE_MEAS);
  MODULE_VOLTAGE_FLOAT =  float(MODULE_VOLTAGE_INT);
  Serial.println(MODULE_VOLTAGE_FLOAT);
  MODULE_VOLTAGE_FLOAT = MODULE_VOLTAGE_INT*(1.0/1023.0);
  MODULE_VOLTAGE_SCALED = MODULE_VOLTAGE_FLOAT*(21.0/1.0);
  Serial.println(MODULE_VOLTAGE_INT);
  Serial.println(MODULE_VOLTAGE_FLOAT);
  Serial.println(MODULE_VOLTAGE_SCALED);  
}

void FADE_TO_COLOR_OF_TEMPERATURE(int RED_LED_INTENSITY_INT, int GREEN_LED_INTENSITY_INT, int BLUE_LED_INTENSITY_INT){
  int TARGET_RED_LED_INTENSITY_INT = GET_RED_LED_INTENSITY(TEMPERATURE_INT, TEMPERATURE_FLOAT);
  int TARGET_GREEN_LED_INTENSITY_INT = GET_GREEN_LED_INTENSITY(TEMPERATURE_INT, TEMPERATURE_FLOAT);
  int TARGET_BLUE_LED_INTENSITY_INT = GET_BLUE_LED_INTENSITY(TEMPERATURE_INT, TEMPERATURE_FLOAT);

  bool RED_LED_INTENSITY_FADED = false;
  bool GREEN_LED_INTENSITY_FADED = false;
  bool BLUE_LED_INTENSITY_FADED = false;

  while((!RED_LED_INTENSITY_FADED) || (!GREEN_LED_INTENSITY_FADED) || (!BLUE_LED_INTENSITY_FADED)){
    if (RED_LED_INTENSITY_INT < TARGET_RED_LED_INTENSITY_INT){
      RED_LED_INTENSITY_INT = RED_LED_INTENSITY_INT + 1;
    }
  
    else if (RED_LED_INTENSITY_INT > TARGET_RED_LED_INTENSITY_INT){
      RED_LED_INTENSITY_INT = RED_LED_INTENSITY_INT - 1;
    }
  
    else if (RED_LED_INTENSITY_INT == TARGET_RED_LED_INTENSITY_INT){
      RED_LED_INTENSITY_FADED = true;
    }

    if (GREEN_LED_INTENSITY_INT < TARGET_GREEN_LED_INTENSITY_INT){
      GREEN_LED_INTENSITY_INT = GREEN_LED_INTENSITY_INT + 1;
    }
  
    else if (GREEN_LED_INTENSITY_INT > TARGET_GREEN_LED_INTENSITY_INT){
      GREEN_LED_INTENSITY_INT = GREEN_LED_INTENSITY_INT - 1;
    }
  
    else if (GREEN_LED_INTENSITY_INT == TARGET_GREEN_LED_INTENSITY_INT){
      GREEN_LED_INTENSITY_FADED = true;
    }

    if (BLUE_LED_INTENSITY_INT < TARGET_BLUE_LED_INTENSITY_INT){
      BLUE_LED_INTENSITY_INT = BLUE_LED_INTENSITY_INT + 1;
    }
  
    else if (BLUE_LED_INTENSITY_INT > TARGET_BLUE_LED_INTENSITY_INT){
      BLUE_LED_INTENSITY_INT = BLUE_LED_INTENSITY_INT - 1;
    }
  
    else if (BLUE_LED_INTENSITY_INT == TARGET_BLUE_LED_INTENSITY_INT){
      BLUE_LED_INTENSITY_FADED = true;
    }

    LOAD_RGB_LED(RED_LED_INTENSITY_INT, GREEN_LED_INTENSITY_INT, BLUE_LED_INTENSITY_INT);    
    delay(10);
  }
}

void FADE_TO_RANDOM_COLOR(int RED_LED_INTENSITY_INT, int GREEN_LED_INTENSITY_INT, int BLUE_LED_INTENSITY_INT){
  int TARGET_RED_LED_INTENSITY_INT = GET_RANDOM_RED_LED_INTENSITY();
  int TARGET_GREEN_LED_INTENSITY_INT = GET_RANDOM_GREEN_LED_INTENSITY();
  int TARGET_BLUE_LED_INTENSITY_INT = GET_RANDOM_BLUE_LED_INTENSITY();

  bool RED_LED_INTENSITY_FADED = false;
  bool GREEN_LED_INTENSITY_FADED = false;
  bool BLUE_LED_INTENSITY_FADED = false;

  while((!RED_LED_INTENSITY_FADED) || (!GREEN_LED_INTENSITY_FADED) || (!BLUE_LED_INTENSITY_FADED)){
    if (RED_LED_INTENSITY_INT < TARGET_RED_LED_INTENSITY_INT){
      RED_LED_INTENSITY_INT = RED_LED_INTENSITY_INT + 1;
    }
  
    else if (RED_LED_INTENSITY_INT > TARGET_RED_LED_INTENSITY_INT){
      RED_LED_INTENSITY_INT = RED_LED_INTENSITY_INT - 1;
    }
  
    else if (RED_LED_INTENSITY_INT == TARGET_RED_LED_INTENSITY_INT){
      RED_LED_INTENSITY_FADED = true;
    }

    if (GREEN_LED_INTENSITY_INT < TARGET_GREEN_LED_INTENSITY_INT){
      GREEN_LED_INTENSITY_INT = GREEN_LED_INTENSITY_INT + 1;
    }
  
    else if (GREEN_LED_INTENSITY_INT > TARGET_GREEN_LED_INTENSITY_INT){
      GREEN_LED_INTENSITY_INT = GREEN_LED_INTENSITY_INT - 1;
    }
  
    else if (GREEN_LED_INTENSITY_INT == TARGET_GREEN_LED_INTENSITY_INT){
      GREEN_LED_INTENSITY_FADED = true;
    }

    if (BLUE_LED_INTENSITY_INT < TARGET_BLUE_LED_INTENSITY_INT){
      BLUE_LED_INTENSITY_INT = BLUE_LED_INTENSITY_INT + 1;
    }
  
    else if (BLUE_LED_INTENSITY_INT > TARGET_BLUE_LED_INTENSITY_INT){
      BLUE_LED_INTENSITY_INT = BLUE_LED_INTENSITY_INT - 1;
    }
  
    else if (BLUE_LED_INTENSITY_INT == TARGET_BLUE_LED_INTENSITY_INT){
      BLUE_LED_INTENSITY_FADED = true;
    }

    LOAD_RGB_LED(RED_LED_INTENSITY_INT, GREEN_LED_INTENSITY_INT, BLUE_LED_INTENSITY_INT);    
    delay(10);
  }
}

void CHANGE_TO_WIFI_MODE(CommandParameter &Parameters){
  CURRENT_MODE = "WIFI MODE";
}

void CHANGE_TO_SET_TEMP_MODE(CommandParameter &Parameters){
  CURRENT_MODE = "SET TEMP MODE";
}

void CHANGE_TO_SET_RGB_MODE(CommandParameter &Parameters){
  CURRENT_MODE = "SET RGB MODE";
}

void CHANGE_TO_RANDOM_MODE(CommandParameter &Parameters){
  CURRENT_MODE = "RANDOM MODE";
}

void CHANGE_TO_WHITE_MODE(CommandParameter &Parameters){
  CURRENT_MODE = "WHITE MODE";
}

void SET_TEMPERATURE(CommandParameter &Parameters){
  SET_TEMPERATURE_INT = Parameters.NextParameterAsInteger();
}

void SET_RED_INTENSITY(CommandParameter &Parameters){
  SET_RGB_RED_INTENSITY_INT = Parameters.NextParameterAsInteger();
}

void SET_GREEN_INTENSITY(CommandParameter &Parameters){
  SET_RGB_GREEN_INTENSITY_INT = Parameters.NextParameterAsInteger();
}

void SET_BLUE_INTENSITY(CommandParameter &Parameters){
  SET_RGB_BLUE_INTENSITY_INT = Parameters.NextParameterAsInteger();
}

void UPDATE_GUI(){
  Serial.print("{UI|SET|CURRENT_MODE.Text=");
  Serial.print(CURRENT_MODE);
  Serial.println("}");

  Serial.print("{UI|SET|RED_DUTY_CYCLE_DEC.Text=");
  Serial.print(RED_LED_INTENSITY_INT);
  Serial.println("}");

  Serial.print("{UI|SET|RED_DUTY_CYCLE_PERCENT.Text=");
  Serial.print(RED_LED_INTENSITY_FLOAT/1023.0*100.0);
  Serial.println("}");

  Serial.print("{UI|SET|GREEN_DUTY_CYCLE_DEC.Text=");
  Serial.print(GREEN_LED_INTENSITY_INT);
  Serial.println("}");

  Serial.print("{UI|SET|GREEN_DUTY_CYCLE_PERCENT.Text=");
  Serial.print(GREEN_LED_INTENSITY_FLOAT/1023.0*100.0);
  Serial.println("}");

  Serial.print("{UI|SET|BLUE_DUTY_CYCLE_DEC.Text=");
  Serial.print(BLUE_LED_INTENSITY_INT);
  Serial.println("}");

  Serial.print("{UI|SET|BLUE_DUTY_CYCLE_PERCENT.Text=");
  Serial.print(BLUE_LED_INTENSITY_FLOAT/1023.0*100.0);
  Serial.println("}");

  Serial.print("{UI|SET|TEMPERATURE.Text=");
  Serial.print(TEMPERATURE_FLOAT);
  Serial.println("}");

  Serial.print("{UI|SET|DEBUG.Text=");
  Serial.print("0");
  Serial.println("}");
}
